package com.qmth.sop.business.mapper;

import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.qmth.sop.business.bean.dto.DataPermissionDto;
import com.qmth.sop.business.bean.result.ServiceRegionResult;
import com.qmth.sop.business.bean.result.SysLogResult;
import com.qmth.sop.business.entity.TBServiceRegion;
import com.qmth.sop.common.enums.ServiceStatusEnum;
import org.apache.ibatis.annotations.Param;

import java.util.List;

/**
 * <p>
 * 服务区域规划表 Mapper 接口
 * </p>
 *
 * @author wangliang
 * @since 2023-08-01
 */
public interface TBServiceRegionMapper extends BaseMapper<TBServiceRegion> {
    /**
     * 服务区域规划分页查询
     *
     * @param iPage         分页参数
     * @param serviceUnitId 服务单元id
     * @param leadId        大区经理id
     * @param city          区域城市
     * @param dpr           数据权限
     * @return 分页查询结果
     */
    IPage<ServiceRegionResult> findServiceRegionPage(@Param("iPage") Page<SysLogResult> iPage,
                                                     @Param("serviceUnitId") Long serviceUnitId,
                                                     @Param("leadId") Long leadId,
                                                     @Param("city") String city,
                                                     @Param("serviceStatus") ServiceStatusEnum serviceStatus,
                                                     @Param("dpr") DataPermissionDto dpr);

    /**
     * 根据服务单元id和派单号查询大区经理
     *
     * @param serviceUnitId 服务单元id
     * @param crmNo         派单号
     * @return 大区经理id集合
     */
    List<Long> findRegionLeaderByServiceUnitIdAndCrmNo(@Param("serviceUnitId") Long serviceUnitId,
                                                       @Param("crmNo") String crmNo);
}
