package com.qmth.sop.business.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.sop.business.bean.result.SysDingGroupResult;
import com.qmth.sop.business.entity.SysDingGroup;
import com.qmth.sop.business.entity.SysDingObj;
import com.qmth.sop.business.entity.SysUser;
import com.qmth.sop.business.mapper.SysDingGroupMapper;
import com.qmth.sop.business.service.SysDingGroupService;
import com.qmth.sop.business.service.SysDingObjService;
import com.qmth.sop.common.enums.ServiceStatusEnum;
import com.qmth.sop.common.util.ServletUtil;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.Map;

/**
 * 考勤组表 服务实现.
 *
 * @author: shudonghui
 * @date: 2023-08-02 14:21:11
 * @version: 1.0
 * @email: shudonghui@qmth.com.cn
 * @Company: www.qmth.com.cn
 */
@Service
public class SysDingGroupServiceImpl extends ServiceImpl<SysDingGroupMapper, SysDingGroup> implements SysDingGroupService {

    @Resource
    SysDingObjService sysDingObjService;

    /**
     * 查询列表
     *
     * @param iPage
     * @return
     */
    @Override
    public IPage<SysDingGroupResult> query(IPage<Map> iPage, Long serviceId, ServiceStatusEnum status, Long[] supplierIds, Long[] dingRoleIds, Long[] approveRoleIds, Boolean faceOpen, Long createStartTime, Long createEndTime) {
        IPage<SysDingGroupResult> dingGroupResultIPage = this.baseMapper.query(iPage, serviceId,status,supplierIds,dingRoleIds,approveRoleIds,faceOpen,createStartTime,createEndTime);
        dingGroupResultIPage.getRecords().forEach(e -> {
            e.setDingObjs(sysDingObjService.getList(e.getId()));
        });
        return dingGroupResultIPage;
    }

    /**
     * 新增考勤组表
     *
     * @param sysDingGroup
     */
    @Override
    @Transactional
    public void add(SysDingGroupResult sysDingGroup) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        sysDingGroup.setCreateId(sysUser.getId());
        sysDingGroup.setCreateTime(System.currentTimeMillis());
        sysDingGroup.setEnable(true);
        this.saveOrUpdate(sysDingGroup);
        sysDingGroup.getDingObjs().forEach(e -> {
            e.setDingGroupId(sysDingGroup.getId());
            sysDingObjService.save(e);
        });

    }

    /**
     * 修改考勤组表
     *
     * @param sysDingGroup
     */
    @Override
    @Transactional
    public void update(SysDingGroupResult sysDingGroup) {

        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        sysDingGroup.setUpdateId(sysUser.getId());
        sysDingGroup.setUpdateTime(System.currentTimeMillis());
        this.saveOrUpdate(sysDingGroup);
        sysDingObjService.remove(new QueryWrapper<SysDingObj>().lambda().eq(SysDingObj::getDingGroupId, sysDingGroup.getId()));
        sysDingGroup.getDingObjs().forEach(e -> {
            e.setDingGroupId(sysDingGroup.getId());
            sysDingObjService.save(e);
        });
    }

    /**
     * 删除考勤组表
     *
     * @param id
     */
    @Override
    @Transactional
    public void delete(Long id) {
        sysDingObjService.remove(new QueryWrapper<SysDingObj>().lambda().eq(SysDingObj::getDingGroupId, id));
        this.removeById(id);
    }

}