package com.qmth.sop.business.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.sop.business.bean.dto.UserDto;
import com.qmth.sop.business.bean.result.MenuResult;
import com.qmth.sop.business.cache.CommonCacheService;
import com.qmth.sop.business.entity.*;
import com.qmth.sop.business.mapper.SysUserRoleMapper;
import com.qmth.sop.business.service.SysDingGroupService;
import com.qmth.sop.business.service.SysDingObjService;
import com.qmth.sop.business.service.SysUserRoleService;
import com.qmth.sop.common.enums.DingObjTypeEnum;
import com.qmth.sop.common.enums.ExceptionResultEnum;
import com.qmth.sop.common.enums.RoleTypeEnum;
import com.qmth.sop.common.util.ServletUtil;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 用户和角色关联表 服务实现类
 * </p>
 *
 * @author wangliang
 * @since 2023-07-17
 */
@Service
public class SysUserRoleServiceImpl extends ServiceImpl<SysUserRoleMapper, SysUserRole> implements SysUserRoleService {

    @Resource
    CommonCacheService commonCacheService;

    @Resource
    SysDingGroupService sysDingGroupService;

    @Resource
    SysDingObjService sysDingObjService;

    /**
     * 获取用户菜单
     *
     * @return
     */
    @Override
    public MenuResult listByUserId() {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        return commonCacheService.userMenuCache(sysUser.getId());
    }

    @Override
    public List<SysRole> listRoleByUserId(Long userId) {
        return this.baseMapper.listRoleByUserId(userId);
    }

    @Override
    public boolean userContainsRoles(Long userId, RoleTypeEnum... roleTypes) {
        boolean result = false;
        List<SysRole> sysRoleList = this.listRoleByUserId(userId);
        if (CollectionUtils.isNotEmpty(sysRoleList)) {
            List<RoleTypeEnum> targetRoleTypeList = Arrays.stream(roleTypes).collect(Collectors.toList());
            List<RoleTypeEnum> userRoleTypeList = sysRoleList.stream().map(SysRole::getType).collect(Collectors.toList());
            if (userRoleTypeList.containsAll(targetRoleTypeList)) {
                result = true;
            }
        }
        return result;
    }

    /**
     * 新增用户权限
     *
     * @param sysUser
     * @param roleIds
     */
    @Override
    @Transactional
    public void addUserRolePrivilege(SysUser sysUser, Long[] roleIds) {
        List<SysUserRole> sysUserRoleList = new ArrayList<>();
        for (int i = 0; i < roleIds.length; i++) {
            sysUserRoleList.add(new SysUserRole(sysUser.getId(), roleIds[i]));
        }
        this.saveBatch(sysUserRoleList);
    }

    /**
     * 根据角色id查询用户信息
     *
     * @param roleId
     * @return
     */
    @Override
    public List<SysUserRole> listByRoleId(Long roleId) {
        return this.baseMapper.listByRoleId(roleId);
    }

    /**
     * 根据角色类型查询用户信息
     *
     * @param roleTypeEnums
     * @return
     */
    @Override
    public List<UserDto> userListByRoleType(List<RoleTypeEnum> roleTypeEnums) {
        return this.baseMapper.userListByRoleType(roleTypeEnums);
    }

    /**
     * 根据服务单元查找用户
     *
     * @param serviceId
     * @return
     */
    @Override
    public List<SysUserRole> listByServiceId(Long serviceId) {
        SysDingGroup sysDingGroup = sysDingGroupService.getOne(new QueryWrapper<SysDingGroup>().lambda().eq(SysDingGroup::getServiceId, serviceId));
        Optional.ofNullable(sysDingGroup).orElseThrow(() -> ExceptionResultEnum.SERVICE_DING_GROUP_NO_DATA.exception());

        List<SysDingObj> sysDingObjList = sysDingObjService.list(new QueryWrapper<SysDingObj>().lambda().eq(SysDingObj::getDingGroupId, sysDingGroup.getId()).eq(SysDingObj::getType, DingObjTypeEnum.APPROVE));
        if (CollectionUtils.isEmpty(sysDingObjList)) {
            throw ExceptionResultEnum.SERVICE_DING_APPROVE_NO_DATA.exception();
        }

        List<Long> roleIsList = sysDingObjList.stream().map(s -> s.getRoleId()).collect(Collectors.toList());
        List<SysUserRole> sysUserRoleList = this.list(new QueryWrapper<SysUserRole>().lambda().in(SysUserRole::getRoleId, roleIsList));
        if (CollectionUtils.isEmpty(sysUserRoleList)) {
            throw ExceptionResultEnum.SERVICE_DING_APPROVE_USER_NO_DATA.exception();
        }
        return sysUserRoleList;
    }
}
